MODULE disvert_mod
  USE icosa
  REAL(rstd), SAVE, POINTER :: ap(:)
  REAL(rstd), SAVE, POINTER :: bp(:)
  REAL(rstd), SAVE, POINTER :: presnivs(:)

CONTAINS

  SUBROUTINE init_disvert
  USE disvert_std_mod, ONLY: ap_std=>ap, bp_std=>bp, presnivs_std=>presnivs, init_disvert_std=>init_disvert
  USE disvert_ncar_mod, ONLY: ap_ncar=>ap, bp_ncar=>bp, presnivs_ncar=>presnivs, init_disvert_ncar=>init_disvert
  USE disvert_ncarl30_mod, ONLY: ap_ncarl30=>ap, bp_ncarl30=>bp, presnivs_ncarl30=>presnivs, init_disvert_ncarl30=>init_disvert
  USE disvert_dcmip31_mod, ONLY: ap_dcmip31=>ap, bp_dcmip31=>bp, presnivs_dcmip31=>presnivs, init_disvert_dcmip31=>init_disvert
  USE icosa
  IMPLICIT NONE
    CHARACTER(LEN=255) :: disvert_type = 'std'
    
    CALL getin("disvert",disvert_type)
    
    SELECT CASE (TRIM(disvert_type))
      CASE('std')
    
        CALL init_disvert_std
        ap=>ap_std
        bp=>bp_std
        presnivs=>presnivs_std
      
      CASE ('ncar')

        CALL init_disvert_ncar
        ap=>ap_ncar
        bp=>bp_ncar
        presnivs=>presnivs_ncar

      CASE ('dcmip31')

        CALL init_disvert_dcmip31
        ap=>ap_dcmip31
        bp=>bp_dcmip31
        presnivs=>presnivs_dcmip31

      CASE ('ncarl30')

        CALL init_disvert_ncarl30
        ap=>ap_ncarl30
        bp=>bp_ncarl30
        presnivs=>presnivs_ncarl30
        
      CASE default
        PRINT*,'Bad selector for variable disvert : <', TRIM(disvert_type),"> options are <std>, <ncar>, <ncarl30>" 
        STOP
        
    END SELECT

  END SUBROUTINE init_disvert  
  
  SUBROUTINE write_apbp
  USE icosa
  USE netcdf_mod
  IMPLICIT NONE
    REAL(rstd) :: val(llm)
    INTEGER :: status
    INTEGER :: lev,ilev
    INTEGER :: ncid,levid,ilevid,hyaiid,hybiid,hyamid,hybmid,P0id
    INTEGER :: l
    
    status = NF90_CREATE('apbp.nc', NF90_CLOBBER, ncid)
    status = NF90_DEF_DIM(ncid,'lev',llm,lev)
    status = NF90_DEF_DIM(ncid,'ilev',llm+1,ilev)
    
    status = NF90_DEF_VAR(ncid,'lev',NF90_DOUBLE,(/ lev /),levid)
    status = NF90_PUT_ATT(ncid,levid,"long_name","hybrid level at midpoints (1000*(A+B))")
    status = NF90_PUT_ATT(ncid,levid,"units","Pa")
    status = NF90_PUT_ATT(ncid,levid,"positive","up")
    status = NF90_PUT_ATT(ncid,levid,"standard_name","atmosphere_hybrid_sigma_pressure_coordinate")
    status = NF90_PUT_ATT(ncid,levid,"formula_terms","a: hyam b: hybm p0: P0 ps: PS")

    status = NF90_DEF_VAR(ncid,'ilev',NF90_DOUBLE,(/ ilev /),ilevid)
    status = NF90_PUT_ATT(ncid,ilevid,"long_name","hybrid level at interfaces (1000*(A+B))")
    status = NF90_PUT_ATT(ncid,ilevid,"units","Pa")
    status = NF90_PUT_ATT(ncid,ilevid,"positive","up")
    status = NF90_PUT_ATT(ncid,ilevid,"standard_name","atmosphere_hybrid_sigma_pressure_coordinate")
    status = NF90_PUT_ATT(ncid,ilevid,"formula_terms","a: hyai b: hybi p0: P0 ps: PS")
   
    status = NF90_DEF_VAR(ncid,'hyai',NF90_DOUBLE,(/ ilev /),hyaiid)
    status = NF90_PUT_ATT(ncid,hyaiid,"long_name","hybrid A coefficient at layer interfaces")

    status = NF90_DEF_VAR(ncid,'hybi',NF90_DOUBLE,(/ ilev /),hybiid)
    status = NF90_PUT_ATT(ncid,hybiid,"long_name","hybrid B coefficient at layer interfaces")

    status = NF90_DEF_VAR(ncid,'hyam',NF90_DOUBLE,(/ lev /),hyamid)
    status = NF90_PUT_ATT(ncid,hyamid,"long_name","hybrid A coefficient at midpoint interfaces")

    status = NF90_DEF_VAR(ncid,'hybm',NF90_DOUBLE,(/ lev /),hybmid)
    status = NF90_PUT_ATT(ncid,hybmid,"long_name","hybrid B coefficient at midpoint interfaces")
    
    status = NF90_DEF_VAR(ncid,'P0',NF90_DOUBLE,varid=P0id)

    status = NF90_ENDDEF(ncid)   
    
    PRINT*,ap
    PRINT*,bp
    status=NF90_PUT_VAR(ncid,ilevid, ap(:)+bp(:)*Preff)
    
    DO l=1,llm
      val(l)= 0.5*(ap(l+1)+ap(l)+Preff*(bp(l)+bp(l+1)))
    ENDDO
    
    status=NF90_PUT_VAR(ncid,levid, val)

    status=NF90_PUT_VAR(ncid,hyaiid, ap(:)/Preff)
    status=NF90_PUT_VAR(ncid,hybiid, bp(:))
    
    DO l=1,llm
      val(l)= 0.5*(ap(l+1)+ap(l))
    ENDDO
    status=NF90_PUT_VAR(ncid,hyamid, val(:)/Preff)
   
     DO l=1,llm
      val(l)= 0.5*(bp(l+1)+bp(l))
    ENDDO
    status=NF90_PUT_VAR(ncid,hybmid, val(:))
 
    status=NF90_PUT_VAR(ncid,P0id, Preff)
   
   status=NF90_CLOSE(ncid)
  END SUBROUTINE write_apbp

END MODULE disvert_mod
